from spotlight.api.rule.data_rule_result.asynchronous import (
    async_get_data_rule_result,
    async_get_data_rule_results,
)
from spotlight.api.rule.data_rule_result.synchronous import (
    get_data_rule_result,
    get_data_rule_results,
)



from spotlight.api.rule.data_rule.model import DataRuleRequest


def _get_data_rule_request_info(id: str) -> dict:
    return {"endpoint": f"config/data_rule/{id}"}


def _get_data_rules_request_info() -> dict:
    return {"endpoint": f"config/data_rule"}


def _create_data_rule_request_info(request: DataRuleRequest) -> dict:
    return {"endpoint": f"config/data_rule", "json": request.request_dict()}


def _update_data_rule_request_info(id: str, request: DataRuleRequest) -> dict:
    return {"endpoint": f"config/data_rule/{id}", "json": request.request_dict()}


def _delete_data_rule_request_info(id: str) -> dict:
    return {"endpoint": f"config/data_rule/{id}"}



from typing import Optional

from spotlight.api.data.model import TimeseriesQueryRequest, DistinctQueryRequest


def _query_timeseries_request_info(request: TimeseriesQueryRequest) -> dict:
    return {"endpoint": f"data/v1.1/timeseries", "json": request.request_dict()}


def _query_dataset_csv_request_info(id: str, request: TimeseriesQueryRequest) -> dict:
    return {"endpoint": f"data/v1.1/{id}.csv", "json": request.request_dict()}


def _query_distinct_fields_request_info(
    request: DistinctQueryRequest, use_cache: Optional[bool] = None
) -> dict:
    return {
        "endpoint": f"data/v1.1/distinct",
        "json": request.request_dict(),
        "params": {"cache": str(use_cache)},
    }


def _sdr_health_request_info() -> dict:
    return {"endpoint": f"data/v1.1/sdr/health"}



from typing import List, Optional, Any

import pandas as pd
from requests import Response

from spotlight.api.data.__util import (
    _query_dataset_csv_request_info,
    _query_timeseries_request_info,
    _query_distinct_fields_request_info,
    _sdr_health_request_info,
)
from spotlight.api.data.model import TimeseriesQueryRequest, DistinctQueryRequest
from spotlight.core.common.decorators import data_request
from spotlight.core.common.requests import __get_request, __post_request


def _query_timeseries(request: TimeseriesQueryRequest):
    """
    Query timeseries dataset by timeseries query request.

    Args:
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        Response: Response
    """
    request_info = _query_timeseries_request_info(request)
    return __post_request(**request_info)


def _query_distinct_fields(
    request: DistinctQueryRequest, use_cache: Optional[bool] = None
):
    """
    Query dataset for distinct values of a specified field.

    Args:
        request (DistinctQueryRequest): Distinct query request
        use_cache (Optional[bool]): Flag to determine whether data cache should be used in fetching unique values

    Returns:
        Response: Response
    """
    request_info = _query_distinct_fields_request_info(request, use_cache)
    return __post_request(**request_info)


@data_request
def query_timeseries(request: TimeseriesQueryRequest) -> List[Any]:
    """
    Query timeseries dataset by timeseries query request.

    Args:
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        pd.DataFrame: Timeseries DataFrame
    """
    return _query_timeseries(request)


def query_dataset_csv(id: str, request: TimeseriesQueryRequest) -> Response:
    """
    Query dataset CSV by ID.

    Args:
        id (str): Dataset ID
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        Response: HTTP response object
    """
    request_info = _query_dataset_csv_request_info(id, request)
    return __get_request(**request_info)


@data_request
def query_distinct_fields(
    request: DistinctQueryRequest, use_cache: Optional[bool] = None
) -> List[str]:
    """
    Query dataset for distinct values of a specified field.

    Args:
        request (DistinctQueryRequest): Distinct query request
        use_cache (Optional[bool]): Flag to determine whether data cache should be used in fetching unique values

    Returns:
        pd.DataFrame: Timeseries DataFrame
    """
    return _query_distinct_fields(request, use_cache)


@data_request
def sdr_health() -> List[dict]:
    """
    Get SDR Data health.

    Returns:
        List[dict]: Data response
    """
    request_info = _sdr_health_request_info()
    return __get_request(**request_info)



from typing import List, Optional, Any

import pandas as pd
from requests import Response

from spotlight.api.data.__util import (
    _query_dataset_csv_request_info,
    _query_timeseries_request_info,
    _query_distinct_fields_request_info,
    _sdr_health_request_info,
)
from spotlight.api.data.model import TimeseriesQueryRequest, DistinctQueryRequest
from spotlight.core.common.decorators import async_data_request
from spotlight.core.common.requests import __async_get_request, __async_post_request


async def _async_query_timeseries(request: TimeseriesQueryRequest):
    """
    Asynchronously query timeseries dataset by timeseries query request.

    Args:
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        Response: Response
    """
    request_info = _query_timeseries_request_info(request)
    return await __async_post_request(**request_info)


async def _async_query_distinct_fields(
    request: DistinctQueryRequest, use_cache: Optional[bool] = None
):
    """
    Asynchronously query dataset for distinct values of a specified field.

    Args:
        request (DistinctQueryRequest): Distinct query request
        use_cache (Optional[bool]): Flag to determine whether data cache should be used in fetching unique values

    Returns:
        Response: Response
    """
    request_info = _query_distinct_fields_request_info(request, use_cache)
    return await __async_post_request(**request_info)


@async_data_request
async def async_query_timeseries(request: TimeseriesQueryRequest) -> List[Any]:
    """
    Asynchronously query timeseries dataset by timeseries query request.

    Args:
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        pd.DataFrame: Timeseries DataFrame
    """
    return await _async_query_timeseries(request)


async def async_query_dataset_csv(id: str, request: TimeseriesQueryRequest) -> Response:
    """
    Asynchronously query dataset CSV by ID.

    Args:
        id (str): Dataset ID
        request (TimeseriesQueryRequest): Timeseries query request

    Returns:
        Response: HTTP response object
    """
    request_info = _query_dataset_csv_request_info(id, request)
    return await __async_get_request(**request_info)


@async_data_request
async def async_query_distinct_fields(
    request: DistinctQueryRequest, use_cache: Optional[bool] = None
) -> List[str]:
    """
    Asynchronously query dataset for distinct values of a specified field.

    Args:
        request (DistinctQueryRequest): Distinct query request
        use_cache (Optional[bool]): Flag to determine whether data cache should be used in fetching unique values

    Returns:
        pd.DataFrame: Timeseries DataFrame
    """
    return await _async_query_distinct_fields(request, use_cache)


@async_data_request
async def async_sdr_health() -> List[dict]:
    """
    Asynchronously get SDR Data health.

    Returns:
        List[dict]: Data response
    """
    request_info = _sdr_health_request_info()
    return await __async_get_request(**request_info)



"""
Data classes for API requests.
"""

from typing import List, Optional, Any

from pydantic import Field

from spotlight.core.common.base import Base
from spotlight.core.common.enum import Order, ComparisonOperator, LogicalOperator


class Sort(Base):
    field: str
    order: Order


class Filter(Base):
    field: str
    operator: ComparisonOperator
    value: Any


class WhereClause(Base):
    filter: Optional[Filter] = Field(default=None)
    operator: Optional[LogicalOperator] = Field(default=None)
    left: Optional["WhereClause"] = Field(default=None)
    right: Optional["WhereClause"] = Field(default=None)


class TimeseriesQueryRequest(Base):
    id: Optional[str] = Field(default=None)
    dataset_name: Optional[str] = Field(default=None)
    reference_name: Optional[str] = Field(default=None)
    page: Optional[int] = Field(default=None)
    limit: Optional[int] = Field(default=None)
    fields: Optional[List[str]] = Field(default=None)
    sort: Optional[List[Sort]] = Field(default=None)
    where: Optional[WhereClause] = Field(default=None)


class DistinctQueryRequest(Base):
    id: Optional[str] = Field(default=None)
    dataset_name: Optional[str] = Field(default=None)
    reference_name: Optional[str] = Field(default=None)
    field: str = Field(default=None)
    sort: Optional[List[Sort]] = Field(default=None)
    where: Optional[WhereClause] = Field(default=None)



from typing import Optional


def _get_data_rule_result_request_info(id: str) -> dict:
    return {"endpoint": f"config/data_rule_result/{id}"}


def _get_data_rule_results_request_info(data_rule_id: Optional[str] = None) -> dict:
    endpoint = "config/data_rule_result"
    if data_rule_id:
        endpoint += f"?data_rule_id={data_rule_id}"
    return {"endpoint": endpoint}



from typing import Dict, Any, List, Optional

from spotlight.api.rule.data_rule_result.__util import (
    _get_data_rule_result_request_info,
    _get_data_rule_results_request_info,
)
from spotlight.core.common.decorators import data_request
from spotlight.core.common.requests import (
    __get_request,
)


@data_request()
def get_data_rule_result(id: str) -> Dict[str, Any]:
    """
    Get data rule result by ID.

    Args:
        id (str): Data rule result ID

    Returns:
        Dict[str, Any]: Data rule result responses
    """
    request_info = _get_data_rule_result_request_info(id)
    return __get_request(**request_info)


@data_request()
def get_data_rule_results(data_rule_id: Optional[str] = None) -> List[Dict[str, Any]]:
    """
    Get all data rule results or results for a specific data rule if data_rule_id is provided.

    Args:
        data_rule_id (Optional[str]): The ID of the specific data rule to get results for. Default is None.

    Returns:
        List[Dict[str, Any]]: List of data rule result responses
    """
    request_info = _get_data_rule_results_request_info(data_rule_id)
    return __get_request(**request_info)
